<?php

namespace Teknisa\SM\Service;

use Teknisa\Libs\Util\ConnectionCustom;
use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\IntegracaoCpFactory;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\DateUtil;

class IntegracaoCpService {
    
    protected $integracaofactory;
    protected $newCode;
    protected $connectionCustom;
    protected $environment;
    protected $connection;
    protected $entityManager;
    
    public function __construct(ConnectionCustom $connectionCustom, Environment $environment, EntityManager $entityManager){
        $this->connectionCustom  = $connectionCustom;
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
    }
    /**
     * Efetua as configurações necessárias para o usuário do CP
     * salvar a parcela
     * @params array $rows dados da proposta recebidos do CP
    **/
    public function gerarParcelaExtra($rows){
        try{
            $nrorg = $rows["NRORG"];
            $entityManagerConnection = $this->connectionCustom->updateVpdConnection($nrorg);
            
            if ($entityManagerConnection !== false) 
                $this->entityManager = $entityManagerConnection;
            
            $this->connection        = $this->entityManager->getConnection();
            $this->newCode           = new NewCode($this->connection, "NOVO_CODIGO_SC");
            $this->integracaofactory = new IntegracaoCpFactory($this->entityManager);
            
            $retorno = $this->criarParcelaExtraParaContrato($rows, $nrorg);
            return $retorno;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Busca as informações e gera a parcela
     * @params array $rows dados da proposta recebidos do CP
    **/
    public function criarParcelaExtraParaContrato($rows, $nrorg){
        try{
            $this->connection->beginTransaction();
            
            $faturacont     = array();
            $nrseqcontrato  = $rows["NRSEQCONTRATO"];
            $servicos       = $rows["SERVICOS"];
            $cdoperador     = $rows["CDOPERADOR"];
            
            $negociacoes = $this->getDadosNegociacao($nrseqcontrato, $servicos, $nrorg);
            foreach($negociacoes as $negociacao) {
                $vrtotalcomdesc = 0;
                $nrnegociacao   = $negociacao["NRNEGOCIACAOCONTRATO"];
                $itens = $this->getItensParcela($rows, $nrnegociacao, $nrseqcontrato, $nrorg, $cdoperador);
                foreach($itens as &$item) 
                    $vrtotalcomdesc = $vrtotalcomdesc + $item["VRVENCIMENTO"];
                
                if($vrtotalcomdesc > 0) {
                    $dataInicial = DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual());
                    $dataFinal   = DateUtil::getUltimoDiaDoMes($dataInicial);
                    if($negociacao["IDTPFATURA"] == 'P') 
                        $dadosVencimento = $this->getDadosVencimentoParcelamento($negociacao["NRTIPOORCAMENTO"], $nrnegociacao, $nrorg);
                    else
                        $dadosVencimento = $this->getDadosVencimentoPeriodico($nrnegociacao, $nrorg);
                    
                    $datadvencimento = null;
                    if($dadosVencimento["IDTPCALCVENC"] != '1') {
                        $nrdiasvencimento = $dadosVencimento["NRDIACALCVENC"];
                        $datadvencimento  = null;
                        $qtdiasfaturavenc = null;
                    } else {
                        $nrdiasvencimento = null;
                        $qtdiasfaturavenc = $dadosVencimento["QTDIASFATUVENC"]; 
                        
                        if($dadosVencimento["IDMESCALCVENC"] == '1')
                            $datadvencimento = DateUtil::getDataDeString(str_pad($dadosVencimento["NRDIACALCVENC"], 2, "0", STR_PAD_LEFT) . "/" . (int) $dataFinal->format("m") - 1 . "/" . $dataFinal->format("Y"));
                        else if($dadosVencimento["IDMESCALCVENC"] == '2')
                            $datadvencimento = DateUtil::getDataDeString(str_pad($dadosVencimento["NRDIACALCVENC"], 2, "0", STR_PAD_LEFT) . "/" . (int) $dataFinal->format("m") . "/" . $dataFinal->format("Y"));
                        else 
                            $datadvencimento = DateUtil::getDataDeString(str_pad($dadosVencimento["NRDIACALCVENC"], 2, "0", STR_PAD_LEFT) . "/" . (int) $dataFinal->format("m") + 1 . "/" . $dataFinal->format("Y"));
                    }
                    
                    $nrfaturacont = $this->newCode->getCodeByTableName("FCT_FATURACONT", $nrorg, 1, 10, true);
                    foreach($itens as &$item) 
                        $item["NRFATURACONT"] = $nrfaturacont;
                    
                    unset($item);
                    
                    $params = array(
                        "NRFATURACONT"     => $nrfaturacont,
                        "CDFILIALFATURA"   => $negociacao["CDFILIALFATURA"],
                        "NRPARCFATURA"     => $negociacao["NRPARCFATURA"],
                        "IDTPFATURA"       => $negociacao["IDTPFATURA"],
                        "CDTIPOOPERFATURA" => $negociacao["CDTIPOOPERFATURA"],
                        "CDSERIENF"        => $negociacao["CDSERIENF"],
                        "CDEMPRESA"        => $negociacao["CDEMPRESA"],
                        "CDINSCESTA"       => $negociacao["CDINSCESTA"],
                        "CDINSCMUNI"       => $negociacao["CDINSCMUNI"],
                        "IDSITUACAO"       => '1',
                        "QTDIASFATUVENC"   => $qtdiasfaturavenc,
                        "NRDIASVENCIMENTO" => $nrdiasvencimento,
                        "DTINICOMPET"      => $dataInicial->format('d/m/Y'),
                        "DTFINCOMPET"      => $dataFinal->format('d/m/Y'),
                        "DTLIMITEAPU"      => DateUtil::adicionaIntervalo($dataFinal, $dadosVencimento["QTDIASAPURACAO"])->format('d/m/Y'),
                        "DTVENCIMENTO"     => (!$datadvencimento) ? null : $datadvencimento->format('d/m/Y'),
                        "VRVENCIMENTO"     => $vrtotalcomdesc,
                        "VRFALTAFATURAR"   => $vrtotalcomdesc,
                        "NRORG"            => $nrorg,
                        "CDOPERINCLUSAO"   => $cdoperador
                    ); 
                    $this->integracaofactory->inserirParcela($params);
                    $this->inserirRelacionamentoParcelaContrato($nrfaturacont, $nrseqcontrato, $nrorg, $cdoperador);
                    foreach($itens as $item)
                        $this->integracaofactory->inserirItensParcela($item);
                    
                    array_push($faturacont, $nrfaturacont);
                }
            }
            
            $this->connection->commit();
            return $faturacont;
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }    
    }
    /**
     * Retorna os dados da negociação ligados ao elemento alterado
    **/
    public function getDadosNegociacao($nrseqcontrato, $servicos, $nrorg) {
        $idelementos = array();
        $cdservorca  = array();
        foreach($servicos as $servico) {
            array_push($idelementos, $servico["IDELEMENTO"]);
            array_push($cdservorca, $servico["CDSERVORCA"]);
        }
        
        $idelementos = implode("','", $idelementos);
        $cdservorca  = implode(",", $cdservorca);
        
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );

        return $this->integracaofactory->getDadosNegociacao($params, $idelementos, $cdservorca);
    }
    /**
     * Retorna os dados do vencimento dos serviços de tipo de faturamento
     * igual a parcelamento
    **/
    public function getDadosVencimentoParcelamento($nrtipoorcamento, $nrnegociacao, $nrorg) {
        $params = array(
            "NRTIPOORCAMENTO"      => $nrtipoorcamento,
            "NRNEGOCIACAOCONTRATO" => $nrnegociacao,
            "NRORG"                => $nrorg
        );
        $dataSet = $this->integracaofactory->getDadosVencimentoItEscopo($params);
        if(count($dataSet) == 0) {
            $params = array(
                "NRTIPOORCAMENTO"      => $nrtipoorcamento,
                "NRORG"                => $nrorg
            );
            $dataSet = $this->integracaofactory->getDadosVencimentoEscopo($params);
        }
        return $dataSet[0];
    }
    /**
     * Retorna os dados de uma competência da negociação
    **/
    public function getDadosVencimentoPeriodico($nrnegociacao, $nrorg) {
        $params = array(
            "NRNEGOCIACAOCONTRATO" => $nrnegociacao,
            "NRORG"                => $nrorg
        );
        $dataSet = $this->integracaofactory->getDadosCompetencia($params);
        return (count($dataSet) > 0) ? $dataSet[0] : null;
    }
    /**
     * Retorna os dados do servico
    **/
    public function getDadosServico($nrseqcontrato, $cdservorca, $idelemento, $nrorg) {
        $params = array(
            "CDSERVORCA"    => $cdservorca,
            "NRSEQCONTRATO" => $nrseqcontrato,
            "IDELEMENTO"    => $idelemento,
            "NRORG"         => $nrorg
        );
        $dataSet = $this->integracaofactory->getDadosServico($params);
        return count($dataSet) > 0 ? $dataSet[0] : null;
    }
    /**
     * Retorna os dados do tipo de medida
    **/
    public function getDadosTipoMedida($nrseqcontrato, $idelemento, $nrorg) {
        $params = array(
            "IDELEMENTORAIZ" => $idelemento,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRORG"          => $nrorg
        );
        $dataSet = $this->integracaofactory->getDadosTipoMedida($params);
        return count($dataSet) > 0 ? $dataSet[0] : null;
    }
    /**
     * Retorna os dados do servico
    **/
    public function getDadosVariacaoPreco($nrseqcontrato, $cdservorca, $nrorg, $medidavar, $qtmedida) {
        $params = array(
            "CDTPMEDIDAVARIAVEL" => $medidavar,
            "CDSERVORCA"         => $cdservorca,
            "NRSEQCONTRATO"      => $nrseqcontrato,
            "QTMEDIDAORCA"       => $qtmedida,
            "NRORG"              => $nrorg
        );
        return $this->integracaofactory->getDadosVariacaoPreco($params)[0];
    }
    /**
     * Manda inserir na tabela FCT_RELACAGRUPCONTFAT o realionamento entre parcela e contrato
    **/
    public function inserirRelacionamentoParcelaContrato($nrfaturacont, $nrseqcontrato, $nrorg, $cdoperador) {
        $nrrelaciona = $this->newCode->getCodeByTableName("FCT_RELACAGRUPCONTFAT", $nrorg, 1, 10, true);
        $params = array(
            "NRAGRUPCONTFAT" => $nrrelaciona,
            "NRFATURACONT"   => $nrfaturacont,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRORG"          => $nrorg,
            "CDOPERADOR"     => $cdoperador
        );
        $this->integracaofactory->inserirRelacionamentoParcelaContrato($params);
    }
    /**
     *  Busca nos itens da negociação os itens da parcela
    **/
    public function getItensParcela($rows, $nrnegociacao, $nrseqcontrato, $nrorg, $cdoperador) {
        $itensParaInserir = array();
        foreach($rows["SERVICOS"] as $servicoNegociacao) {
            $cdservorca = $servicoNegociacao["CDSERVORCA"];
            $elemento   = $servicoNegociacao["IDELEMENTO"];
            $params = array(
                "NRNEGOCIACAOCONTRATO" => $nrnegociacao,
                "CDSERVORCA"           => $cdservorca,
                "IDELEMENTO"           => $elemento,
                "NRORG"                => $nrorg
            );
            
            $item = $this->integracaofactory->getItensNegociacaoContrato($params);
            if(count($item) > 0) {
                $servicos = $this->getDadosServico($nrseqcontrato, $cdservorca, $elemento, $nrorg);
                $medidas  = $this->getDadosTipoMedida($nrseqcontrato, $elemento, $nrorg);
                
                $medidavar   = ($medidas && count($medidas) > 0) ? $medidas["CDTPMEDIDAVARIAVEL"] : $servicos["CDTPMEDIDASERV"];
                $qtmedidavar = $servicoNegociacao["QTMEDIDAORCA"];
                $variacoes   = $this->getDadosVariacaoPreco($nrseqcontrato, $cdservorca, $nrorg, $medidavar, $qtmedidavar);

                $vrtotalcomdesc = ($variacoes["VRPRECOVARIACAO"] * $servicos["QTMEDIDASERV"]);
                $nritfaturacont = $this->newCode->getCodeByTableName("FCT_ITFATURACONT", $nrorg, 1, 10, true);
                $itensParaInserir[] = array(
                    "NRFATURACONT"    => null,
                    "IDSITUACAO"      => '1',
                    "NRITEMOCONTRATO" => $item[0]["NRITEMOCONTRATO"],
                    "CDPRODFATURA"    => $item[0]["CDPRODFATURA"],
                    "NRITFATURACONT"  => $nritfaturacont,
                    "NRSEQCONTRATO"   => $nrseqcontrato,
                    "VRVENCIMENTO"    => $vrtotalcomdesc,
                    "VRFALTAFATURAR"  => $vrtotalcomdesc,
                    "NRORG"           => $nrorg,
                    "CDOPERADOR"      => $cdoperador
                );
            }
        }
        return $itensParaInserir;
    }
}
